/******************************************************************************
 * ctcc.c
 * SASS Compiler for CoreTone.
 *-----------------------------------------------------------------------------
 * Version 1.2.2cz, November 26th, 2016
 * Copyright (C) 2015 - 2016 Osman Celimli
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 ******************************************************************************/
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <math.h>

#include "../types.h"
#include "../CoreTone/sample.h"
#include "../CoreTone/channel.h"
#include "../CoreTone/music.h"

#include "sample.h"
#include "macro.h"
#include "music.h"

/******************************************************************************
 * Modes
 ******************************************************************************/
typedef enum MakeMode
{
	eMODE_SMP = 0,
	eMODE_SFX,
	eMODE_INS,
	eMODE_MUS,

	eMODE_FOOTER
} MakeMode;

struct
{
	char szName[5];

	MakeMode eMode;
	int32_t iFixedArgs;
	uint32_t uiMinArgs;

	int32_t iAutoWrite;
	uint32_t uiOutFile;
	uint32_t uiFirstIn;
} aModes[] = {
	{"-smp", eMODE_SMP, -1, 4, -1, 2, 3},
	{"-sfx", eMODE_SFX, -1, 4,  0, 0, 2},
	{"-ins", eMODE_INS, -1, 5, -1, 2, 3},
	{"-mus", eMODE_MUS,  0, 5, -1, 2, 3}
};

/******************************************************************************
 * Text Tables
 ******************************************************************************/
const char szERR_Usage[]=
	"Usage :\n"
	"  ctcc (-smp/-sfx/-ins/-mus) [outfile] (source data)\n"
	" Where the specific syntax depends upon the operating mode, all of which are\n"
	" listed below...\n"
	"\n"
	"::Sample Pack Mode (-smp),\n"
	"  ctcc -smp output.smp samples.txt\n"
	" All samples in the sample descriptor file must be stored in 8-Bit Signed PCM\n"
	" with a frequency between 1.0Hz and 65535.99Hz\n"
	"\n"
	"::Sound Effect Mode (-sfx),\n"
	"  ctcc -sfx samples.txt sfx_scripts.txt\n"
	" Generated sound effect files (*.sfx) will be stored in the same directory as\n"
	" sfx_scripts.txt, with one sound effect file per SASS sound effect definition.\n"
	"\n"
	"::Instrument Pack Mode (-ins),\n"
	"  ctcc -ins output.ins samples.txt instruments.txt\n"
	"\n"
	"::Music Track Mode (-mus),\n"
	"  ctcc -mus output.mus instruments.txt channel_0.txt [channel_1.txt] [...]\n"
	" Music written with more channels than available on a particular target will\n"
	" have them dropped during playback rather than at compilation.\n"
	"\n";

const char szREM_SMPMode[] =		"SMP Mode...\n";
const char szREM_SFXMode[] =		"SFX Mode...\n";
const char szREM_INSMode[] =		"INS Mode...\n";
const char szREM_MUSMode[] =		"MUS Mode...\n";

const char szERR_NoMainMem[] =		"ERROR- Out of memory\n";
const char szERR_Null[] =			"ERROR- Unexpected NULL.\n";
const char szERR_Range[] =			"ERROR- Value out of range\n";
const char szERR_TooManyFiles[] =	"ERROR- Too many source files.\n";
const char szERR_NotDone[] =		"ERROR- Feature not implemented.\n";
const char szERR_What[] =			"ERROR- I just don't know what went wrong!\n";

const char szERR_BadMode[] =		"ERROR- Invalid operating mode : ";
const char szERR_Syntax[] =			"ERROR- Invalid command syntax in : ";
const char szERR_EarlyEnd[] =		"ERROR- Unexpected end of source data in : ";
const char szERR_NoOpen[] =			"ERROR- Can't open source file : ";
const char szERR_NoDest[] =			"ERROR- Can't write to destination file : ";

const char szERR_Garbage[] =		"ERROR- Garbage at end of line in : ";




/******************************************************************************
 * int main(int argc, char **argv)
 *
 * Possible argument formats:
 *  ctcc -smp output.smp samples.txt
 *  ctcc -sfx samples.txt sfx_scripts.txt
 *  ctcc -ins output.ins samples.txt instruments.txt
 *  ctcc -mus output.mus samples.txt instruments.txt channel_0.txt [channel_1.txt] [...]
 *----------------------------------------------------------------------------*/
int main(int argc, char **argv)
{
	FILE *pDest = NULL;
	FILE **ppSrcs = NULL;
	uint32_t uiSources;

	int32_t iFailed;

	uint32_t uiX,uiY,uiZ;
	MakeMode eMode;

	/**
	 * Enable all allocation checking for debug buids, I'd rather not
	 * leak memory all over the counter and attract ants.
	 */
	#ifdef _DEBUG
		int tMemDbg;
		_CrtSetReportMode( _CRT_ERROR, _CRTDBG_MODE_DEBUG );
		tMemDbg = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );
		tMemDbg |= _CRTDBG_CHECK_ALWAYS_DF;
		tMemDbg |= _CRTDBG_CHECK_CRT_DF;
		tMemDbg |= _CRTDBG_DELAY_FREE_MEM_DF;
		tMemDbg |= _CRTDBG_LEAK_CHECK_DF;
		_CrtSetDbgFlag( tMemDbg );
	#endif

	/**
	 * Determine our operating mode and perform basic checking on the
	 * supplied arguments from the user. If the detected mode has a fixed
	 * output file slot in the argument list, attempt to open it for writing.
	 */
	if(argc < 2)
	{
		fprintf(stderr, "%s", szERR_Usage);
		return 0;
	}

	eMode = eMODE_FOOTER;
	for(uiX = 0; uiX < eMODE_FOOTER; uiX++)
	{
		if(0 == strcmp(argv[1], aModes[uiX].szName))
		{
			eMode = aModes[uiX].eMode;
			break;
		}
	}
	if(eMODE_FOOTER == eMode)
	{
		fprintf(stderr, "%s%s\n", szERR_BadMode, argv[1]);
		return 0;
	}

	uiZ = argc;
	if((aModes[eMode].iFixedArgs && (uiZ != aModes[eMode].uiMinArgs))
		|| (!(aModes[eMode].iFixedArgs) && (uiZ < aModes[eMode].uiMinArgs)))
	{
		fprintf(stderr, "%s", szERR_Usage);
		return 0;
	}

	if(aModes[eMode].iAutoWrite)
	{
		if(NULL == (pDest = fopen(argv[ aModes[eMode].uiOutFile ],"wb")))
		{
			fprintf(stderr, "%s%s\n", szERR_NoDest,
				argv[ aModes[eMode].uiOutFile ]);
			return 0;
		}
	}

	/**
	 * Attempt to open all of the source files required for the given mode,
	 * it's assumed that all modes will only need text reads as binary
	 * data is only requested from within a SASS script (in particular,
	 * sample descriptors).
	 */
	uiSources = argc - aModes[eMode].uiFirstIn;
	if(NULL == (ppSrcs = malloc(sizeof(FILE) * uiSources)))
	{
		fprintf(stderr, "%s", szERR_NoMainMem);
		goto cleanup;
	}
	for(uiX = 0; uiX < uiSources; uiX++)
		ppSrcs[uiX] = NULL;

	for(uiX = 0; uiX < uiSources; uiX++)
	{
		uiY = aModes[eMode].uiFirstIn + uiX;

		if(NULL == (ppSrcs[uiX] = fopen(argv[uiY], "r") ))
		{
			fprintf(stderr, "%s%s\n", szERR_NoOpen, argv[uiY]);
			goto cleanup;
		}
	}


	/**
	 * Handle the compilation procedure for whichever mode was selected.
	 */
	switch(eMode)
	{
		/* !!!!---- SAMPLE MODE ----!!!! */
		case eMODE_SMP:
			fprintf(stdout, "%s", szREM_SMPMode);

			iFailed = smp_parse(ppSrcs[0], argv[ aModes[eMode].uiFirstIn ]);
			if(iFailed) goto cleanup;
			iFailed = smp_make(pDest, argv[ aModes[eMode].uiOutFile ]);
			break;

		/* !!!!---- SOUND EFFECT MODE ----!!!! */
		case eMODE_SFX:
			fprintf(stdout, "%s", szREM_SFXMode);

			uiX = aModes[eMode].uiFirstIn;
			iFailed = smp_parse(ppSrcs[0], argv[uiX + 0]);
			if(iFailed) goto cleanup;
			iFailed = mac_parse(ppSrcs[1], argv[uiX + 1], -1, 0);
			if(iFailed) goto cleanup;

			iFailed = sfx_make();
			break;

		/* !!!!---- INSTRUMENT MODE ----!!!! */
		case eMODE_INS:
			fprintf(stdout, "%s", szREM_INSMode);

			uiX = aModes[eMode].uiFirstIn;
			iFailed = smp_parse(ppSrcs[0], argv[uiX + 0]);
			if(iFailed) goto cleanup;
			iFailed = mac_parse(ppSrcs[1], argv[uiX + 1], 0, 0);
			if(iFailed) goto cleanup;

			iFailed = ins_make(pDest, argv[ aModes[eMode].uiOutFile ]);
			break;

		/* !!!!---- MUSIC MODE ----!!!! */
		case eMODE_MUS:
			fprintf(stdout, "%s", szREM_MUSMode);

			uiX = aModes[eMode].uiFirstIn;
			uiY = argc;
			iFailed = mac_parse(ppSrcs[0], argv[uiX++], 0, -1);
			if(iFailed) goto cleanup;
			for(uiZ = 1; uiX < uiY; uiZ++, uiX++)
			{
				iFailed = mus_parse(ppSrcs[uiZ], argv[uiX]);
				if(iFailed) goto cleanup;
			}

			iFailed = mus_make(pDest, argv[ aModes[eMode].uiOutFile ]);
			break;

		/* Uh... */
		default:
			fprintf(stderr, "%s", szERR_What);
			break;
	}


cleanup:
	fflush(stdout);
	fflush(stderr);

	mus_close();
	mac_close();
	smp_close();

	if(NULL != pDest)
	{
		fclose(pDest);
		pDest = NULL;
	}
	if(NULL != ppSrcs)
	{
		for(uiX = 0; uiX < uiSources; uiX++)
		{
			if(NULL != ppSrcs[uiX])
			{
				fclose(ppSrcs[uiX]);
				ppSrcs[uiX] = NULL;
			}
		}

		free(ppSrcs);
		ppSrcs = NULL;
	}

	return 0;
}
