/******************************************************************************
 * music.h
 * SASS Compiler for CoreTone, music parsing and track generation.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in ctcc.c
 ******************************************************************************/
#ifndef CTCC_MUSIC
#define CTCC_MUSIC

/******************************************************************************
 * Operating Parameters
 ******************************************************************************/
#define MUSIC_PARSER_LINEBUF_LEN	2048

#define MUSIC_PARSER_BLOCK_LEN		64
#define MUSIC_PARSER_TAG_LEN		64
#define MUSIC_PARSER_TRACK_LEN		4
#define MUSIC_PARSER_STREAM_LEN		1024

#define MUSIC_PARSER_PRIORITY_MIN	-128
#define MUSIC_PARSER_PRIORITY_OFF	0
#define MUSIC_PARSER_PRIORITY_MAX	127
#define MUSIC_PARSER_PANNING_MIN	-128
#define MUSIC_PARSER_PANNING_MAX	127
#define MUSIC_PARSER_LOOP_MIN		-128
#define MUSIC_PARSER_LOOP_MAX		127
#define MUSIC_PARSER_WAIT_MIN		0

/******************************************************************************
 * Music Tracks
 ******************************************************************************/
typedef struct MusicBlock_s
{
	char *pszName;

	int32_t iWasWaiting;
	uint32_t uiDelayAcc,uiDelayLen;

	uint8_t *pacStream;
	uint32_t uiScriptLen,uiBufferLen;
	uint32_t uiOffset;
} MusicBlock_t;

typedef struct MusicTag_s
{
	char *pszName;
	uint32_t uiOffset;
	
	char *pszSrcFile;
	uint32_t uiLine;
} MusicTag_t;

typedef struct MusicTrack_s
{
	int32_t iFoundMain,iMainPriority;
	uint32_t uiMainOffset;
} MusicTrack_t;

/* Track Table
 *------------------------------------*/
extern MusicBlock_t *paMusBlocks;
extern MusicTag_t *paMusTags;
extern MusicTrack_t *paMusTracks;

/******************************************************************************
 * Function Defines
 ******************************************************************************/
int32_t mus_parse(FILE *pSASS, char *pszFileName);
int32_t mus_make(FILE *pDest, char *pszFileName);
void mus_close(void);

int32_t mus_addByte(MusicBlock_t *pBlock, uint8_t ucByte);
int32_t mus_prepBlock(MusicBlock_t *pBlock, char *pszName);
int32_t mus_getBlock(char *pszName, uint32_t *puiNum, MusicBlock_t **ppBlock);
int32_t mus_addCallTag(char *pszTarget, uint32_t uiOffset,
                       char *pszSrcFile, uint32_t uiLine);

#endif
