/******************************************************************************
 * menuman.hpp
 * Dropdown menu manager.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#ifndef MENUMAN
#define MENUMAN

/******************************************************************************
 * Layout
 ******************************************************************************/
/* FILE \
 *  - OPEN SAMPLES
 *  - OPEN MACROS
 * ------------------------
 *  - OPEN MUSIC
 * ------------------------
 *  - LOG OUTPUT
 *  - STOP LOGGING
 * ------------------------
 *  - EXIT
 */
#define MFILE_SLOT						0
#define MFILE_SUBSLOT_OPEN_SAMPLES		0
#define MFILE_SUBSLOT_OPEN_MACROS		1
#define MFILE_SUBSLOT_OPEN_MUSIC		3
#define MFILE_SUBSLOT_LOG_OUTPUT		5
#define MFILE_SUBSLOT_STOP_LOGGING		6
#define MFILE_SUBSLOT_EXIT				8


/* PLAY \
 *  - PAUSE
 *  - RESUME
 * ------------------------
 *  - PLAY MUSIC
 *  - STOP MUSIC
 * ------------------------
 *  - STOP ALL
 */
#define MPLAY_SLOT						1
#define MPLAY_SUBSLOT_PAUSE				0
#define MPLAY_SUBSLOT_RESUME			1
#define MPLAY_SUBSLOT_PLAY_MUSIC		3
#define MPLAY_SUBSLOT_STOP_MUSIC		4
#define MPLAY_SUBSLOT_STOP_ALL			6


/* DEBUG \
 *  - PATCH ACTIVITY
 *  - MUSIC SCRIPTS
 * ------------------------
 *  - SOUND EFFECTS
 */
#define MDEBUG_SLOT						2
#define MDEBUG_SUBSLOT_PATCHES			0
#define MDEBUG_SUBSLOT_MUSIC			1
#define MDEBUG_SUBSLOT_SFX				3


/* HELP \
 *  - ABOUT
 */
#define MHELP_SLOT						3
#define MHELP_SUBSLOT_ABOUT				0


/******************************************************************************
 * Class
 ******************************************************************************/
#define MENUMAN_BUFFER_ENTRIES			8192
#define MENUMAN_BUFFER_LEN				(sizeof(mszBuffer) / sizeof(mszBuffer[0]))

typedef enum FileType_e FileType_t;

class MenuMan {
	public:
		MenuMan(HWND hParent);
		~MenuMan(void);

		BOOL openFile(FileType_t eMode);
		BOOL setupLog(void);

		LRESULT cmdProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		void refresh(void);

		/**
		 * ---- PEEKABLE LOADED FILE NAMES ----
		 */
		TCHAR mszSample[MENUMAN_BUFFER_ENTRIES];
		TCHAR mszMacros[MENUMAN_BUFFER_ENTRIES];
		TCHAR mszMusic[MENUMAN_BUFFER_ENTRIES];
		TCHAR mszSFX[MENUMAN_BUFFER_ENTRIES];

	private:
		/**
		 * ---- INTERNALS AND MENU STATE ----
		 */
		HWND mhParent;

		HMENU mhMain;
		HMENU mhSub_File;
		HMENU mhSub_Play;
		HMENU mhSub_Debug;
		HMENU mhSub_Help;

		OPENFILENAME mOpenParams,mSaveParams;
		TCHAR mszBuffer[MENUMAN_BUFFER_ENTRIES];
		TCHAR mszOpenName[MENUMAN_BUFFER_ENTRIES];
		TCHAR mszSaveName[MENUMAN_BUFFER_ENTRIES];
};

#endif
