/******************************************************************************
 * panelman.hpp
 * Main button panel manager : load, play, pause, stop, record, etc.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#ifndef PANELMAN
#define PANELMAN

/******************************************************************************
 * Layout
 ******************************************************************************/
/* |                                                                          |
 * | [SAMPLE] [MACRO] [MUSIC] [RECORD]  [PLAY] [PAUSE] [STOP]  ------[]------ |
 * |                                                                          |
 */
#define PANELMAN_WIDTH					400
#define PANELMAN_HEIGHT					32


/* Button Arrangement
 *-----------------------------------*/
#define PANELMAN_S_BUTTON_WIDTH			28
#define PANELMAN_W_BUTTON_WIDTH			40
#define PANELMAN_BUTTON_HEIGHT			24

#define PANELMAN_BUTTON_F_SPACING		10
#define PANELMAN_BUTTON_B_SPACING		4
#define PANELMAN_BUTTON_G_SPACING		8

#define PANELMAN_BUTTON_V_SPACING		4


/* Volume Slider
 *-----------------------------------*/
#define PANELMAN_SPEAKER_I_WIDTH		16
#define PANELMAN_SPEAKER_I_HEIGHT		16

#define PANELMAN_SPEAKER_H_SPACING		12
#define PANELMAN_SPEAKER_V_SPACING		8


#define PANELMAN_VOLUME_I_WIDTH			80
#define PANELMAN_VOLUME_I_HEIGHT		24

#define PANELMAN_VOLUME_H_SPACING		8
#define PANELMAN_VOLUME_V_SPACING		4


/* Control IDs
 *-----------------------------------*/
#define PANELMAN_ID_BASE				7000
#define PANELMAN_ID_CONTAINER			(PANELMAN_ID_BASE + 0)

#define PANELMAN_ID_OPEN_SAMPLE			(PANELMAN_ID_BASE + 1)
#define PANELMAN_ID_OPEN_MACRO			(PANELMAN_ID_BASE + 2)
#define PANELMAN_ID_OPEN_MUSIC			(PANELMAN_ID_BASE + 3)
#define PANELMAN_ID_RECORD				(PANELMAN_ID_BASE + 4)

#define PANELMAN_ID_PLAY				(PANELMAN_ID_BASE + 5)
#define PANELMAN_ID_PAUSE				(PANELMAN_ID_BASE + 6)
#define PANELMAN_ID_STOP				(PANELMAN_ID_BASE + 7)

#define PANELMAN_ID_VOL_ICON			(PANELMAN_ID_BASE + 8)
#define PANELMAN_ID_VOL_SLIDER			(PANELMAN_ID_BASE + 9)


/******************************************************************************
 * Class
 ******************************************************************************/
class PanelMan {
	public:
		PanelMan(HWND hParent, INT iX, INT iY);
		~PanelMan(void);

		LRESULT cmdProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		LRESULT trackProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

		HICON getIcon(INT iRes);
		void refresh(void);

	private:
		HWND mhParent;
		INT miX,miY;

		HWND mhSpeaker,mhVolume;
		DWORD mdwVol;

		Icon_t *mpIcons;
		Button_t *mpButtons;
};

#endif
